from sota_extractor.scrapers.utils import get_soup
from sota_extractor.taskdb.v01 import (
    Link,
    Task,
    Dataset,
    Sota,
    SotaRow,
    TaskDB,
)

SNLI_URL = "https://nlp.stanford.edu/projects/snli/"


def snli():
    """Extract SNLI SOTA tables."""
    soup = get_soup(SNLI_URL)

    table = soup.findAll("table", attrs={"class": "newstuff"})[1]

    rows = table.findAll("tr")

    sota_rows = []
    # suffix = ""
    for row in rows:
        # ignore the header
        if row.get("class") == ["header"]:
            pass
        elif row.get("class") == ["section"]:
            # suffix = row.text.replace("models", "").strip()
            continue
        else:
            cells = row.findAll("td")

            a = cells[0].find("a")

            paper_url = a.get("href")
            if paper_url == "http://nlp.stanford.edu/pubs/snli_paper.pdf":
                paper_title = (
                    "A large annotated corpus for learning natural language "
                    "inference"
                )
            elif paper_url == "https://www.nyu.edu/projects/bowman/spinn.pdf":
                paper_title = (
                    "A Fast Unified Model for Parsing and Sentence "
                    "Understanding"
                )
            elif (
                paper_url
                == "https://s3-us-west-2.amazonaws.com/openai-assets/"
                "research-covers/language-unsupervised/"
                "language_understanding_paper.pdf"
            ):
                paper_title = (
                    "Improving Language Understanding by Generative "
                    "Pre-Training"
                )
            elif (
                paper_url == "https://pdfs.semanticscholar.org/adc1/"
                "84fcb04107f95e35ea1b07ef9aad749da8d7.pdf"
            ):
                paper_title = "Deep Fusion LSTMs for Text Semantic Matching"
            else:
                paper_title = a.text

            model_name = cells[1].text.strip()
            # if suffix:
            #    model_name = "%s (%s)" % (model_name, suffix)

            model_name = model_name.replace("(code)", "").strip()

            params = cells[2].text.strip()
            train_acc = cells[3].text.strip()
            test_acc = cells[4].text.strip()

            sota_rows.append(
                SotaRow(
                    model_name=model_name,
                    paper_title=paper_title,
                    paper_url=paper_url,
                    metrics={
                        "% Test Accuracy": test_acc,
                        "% Train Accuracy": train_acc,
                        "Parameters": params,
                    },
                )
            )

    task = Task(
        name="Natural Language Inference",
        datasets=[
            Dataset(
                name="SNLI",
                is_subdataset=False,
                sota=Sota(
                    metrics=[
                        "% Test Accuracy",
                        "% Train Accuracy",
                        "Parameters",
                    ],
                    rows=sota_rows,
                ),
            )
        ],
        source_link=Link(
            title="The Stanford Natural Language Inference (SNLI) Corpus",
            url="https://nlp.stanford.edu/projects/snli/",
        ),
    )
    tdb = TaskDB()
    tdb.add_task(task)
    return tdb.export()



from copy import deepcopy
from marshmallow import Schema, fields, pre_load, post_load, post_dump
from sota_extractor.taskdb.v01.models import Link, SotaRow, Sota, Dataset, Task


class LinkSchema(Schema):
    title = fields.String(missing="")
    url = fields.String(missing="")

    @post_load
    def post_load(self, data):
        return Link(**data)


class SotaRowSchema(Schema):
    model_name = fields.String(required=True)
    paper_title = fields.String(missing="")
    paper_url = fields.String(missing="")
    paper_date = fields.Date(format="%Y-%m-%d", allow_none=True, missing=None)
    code_links = fields.Nested(LinkSchema, many=True, missing=list)
    model_links = fields.Nested(LinkSchema, many=True, missing=list)
    metrics = fields.Dict(keys=fields.String(), values=fields.Inferred())

    @post_load
    def post_load(self, data):
        return SotaRow(**data)


class SotaSchema(Schema):
    metrics = fields.List(fields.String(), missing=list)
    rows = fields.Nested(SotaRowSchema, many=True, missing=list)

    @post_load
    def post_load(self, data):
        return Sota(**data)


class DatasetSchema(Schema):
    name = fields.String()
    is_subdataset = fields.Boolean()
    description = fields.String(missing="")
    sota = fields.Nested(SotaSchema, missing=lambda: Sota())
    subdatasets = fields.Nested("self", many=True, missing=list)
    links = fields.Nested(
        LinkSchema, data_key="dataset_links", many=True, missing=list
    )
    citations = fields.Nested(
        LinkSchema, data_key="dataset_citations", many=True, missing=list
    )

    @pre_load
    def pre_load(self, data):
        data = deepcopy(data)
        if "dataset" in data:
            data["name"] = data["dataset"]
            data["is_subdataset"] = False
            del data["dataset"]
        elif "subdataset" in data:
            data["name"] = data["subdataset"]
            data["is_subdataset"] = True
            del data["subdataset"]
        else:
            data["name"] = ""
            data["is_subdataset"] = False
        return data

    @post_load
    def post_load(self, data):
        dataset = Dataset(**data)
        for subdataset in dataset.subdatasets:
            subdataset.parent = dataset
        return dataset

    @post_dump
    def post_dump(self, data):
        if data["is_subdataset"]:
            data["subdataset"] = data["name"]
        else:
            data["dataset"] = data["name"]

        del data["name"]
        del data["is_subdataset"]
        return data


class TaskSchema(Schema):
    name = fields.String(required=True, data_key="task")
    description = fields.String(missing="")
    categories = fields.List(fields.String(), missing=list)
    datasets = fields.Nested(DatasetSchema, many=True, missing=list)
    subtasks = fields.Nested("self", many=True, missing=list)
    synonyms = fields.List(fields.String(), missing=list)
    source_link = fields.Nested(LinkSchema, allow_none=True, missing=None)

    @post_load
    def post_load(self, data):
        task = Task(**data)
        for subtask in task.subtasks:
            subtask.parent = task
        return task



from datetime import datetime
from sota_extractor.errors import DataError
from sota_extractor.scrapers.utils import get_soup
from sota_extractor.taskdb.v01 import SotaRow, Dataset, Task, Link, TaskDB

SQUAD_URL = "https://rajpurkar.github.io/SQuAD-explorer/"


TASK_NAME = "Question Answering"
DATASET_1_NAME = "SQuAD1.1"
DATASET_1_DESCRIPTION = (
    "Previous version of the SQuAD2.0 dataset with 50,000 question-answer "
    "pairs."
)
DATASET_2_NAME = "SQuAD2.0"
DATASET_2_DESCRIPTION = (
    "Stanford Question Answering Dataset (SQuAD) is a reading comprehension "
    "dataset, consisting of questions posed by crowdworkers on a set of "
    "Wikipedia articles, where the answer to every question is a segment of "
    "text, or span, from the corresponding reading passage, or the question "
    "might be unanswerable."
)


def get_sota_rows(table):
    rows = table.findAll("tr")

    sota_rows = []
    for row in rows:
        klass = row.attrs.get("class")
        if klass and klass[0] == "human-row":
            # Skip the human performance row
            continue
        cells = row.findAll("td")
        if len(cells) == 4:
            date_span = cells[0].select_one("span.date")
            if date_span is not None:
                date = datetime.strptime(date_span.text, "%b %d, %Y")
            else:
                date = None
            model = cells[1]
            m_em = cells[2].text
            m_f1 = cells[3].text

            if model.find("a"):
                sota_rows.append(
                    SotaRow(
                        model_name=model.find(text=True, recursive=False),
                        paper_title=model.find("a").text,
                        paper_url=model.find("a")["href"],
                        paper_date=date,
                        metrics={"EM": m_em, "F1": m_f1},
                    )
                )
            else:
                sota_rows.append(
                    SotaRow(
                        model_name=model.find(text=True, recursive=False),
                        paper_date=date,
                        metrics={"EM": m_em, "F1": m_f1},
                    )
                )

    return sota_rows


def squad():
    """Extract SQUAD SOTA tables."""
    soup = get_soup(SQUAD_URL)

    sota_tabels = soup.findAll("table", attrs={"class": "performanceTable"})

    if len(sota_tabels) == 2:
        squad2 = sota_tabels[0]
        squad1 = sota_tabels[1]

        dataset2 = Dataset(
            name=DATASET_2_NAME,
            is_subdataset=False,
            description=DATASET_2_DESCRIPTION,
        )
        dataset1 = Dataset(
            name=DATASET_1_NAME,
            is_subdataset=False,
            description=DATASET_1_DESCRIPTION,
        )

        task = Task(name="Question Answering")
        task.datasets = [dataset2, dataset1]
        task.source_link = Link(title="SQuAD Leaderboard", url=SQUAD_URL)

        # scrape the evaluation values on the two datasets
        dataset2.sota.metrics = ["EM", "F1"]
        dataset1.sota.metrics = ["EM", "F1"]

        dataset2.sota.rows = get_sota_rows(squad2)
        dataset1.sota.rows = get_sota_rows(squad1)

        tdb = TaskDB()
        tdb.add_task(task)
        return tdb.export()
    else:
        raise DataError("Got an unexpected number of SOTA tables.")



from typing import List, Optional, Dict
from dataclasses import dataclass, field


@dataclass
class Link:
    title: str = ""
    url: str = ""


@dataclass
class SotaRow:
    model_name: str
    paper_title: str = ""
    paper_url: str = ""
    paper_date: Optional[str] = None
    code_links: List[Link] = field(default_factory=list)
    model_links: List[Link] = field(default_factory=list)
    metrics: Dict[str, str] = field(default_factory=dict)


@dataclass
class Sota:
    metrics: List[str] = field(default_factory=list)
    rows: List[SotaRow] = field(default_factory=list)


@dataclass
class Dataset:
    name: str
    is_subdataset: bool
    description: str = ""
    parent: "Dataset" = None
    sota: Sota = field(default_factory=lambda: Sota())
    subdatasets: List["Dataset"] = field(default_factory=list)
    links: List[Link] = field(default_factory=list)
    citations: List[Link] = field(default_factory=list)


@dataclass
class Task:
    name: str
    description: str = ""
    parent: "Task" = None
    categories: List[str] = field(default_factory=list)
    datasets: List[Dataset] = field(default_factory=list)
    subtasks: List["Task"] = field(default_factory=list)
    synonyms: List[str] = field(default_factory=list)
    source_link: Link = None



import re
import requests
from bs4 import BeautifulSoup
from sota_extractor.taskdb.v01 import (
    Task,
    Dataset,
    Link,
    Sota,
    SotaRow,
    TaskDB,
)

REDITSOTA_URL = (
    "https://raw.githubusercontent.com/RedditSota/"
    "state-of-the-art-result-for-machine-learning-problems/master/README.md"
)


def reddit():
    """Extract Reddit SOTA tables."""
    tdb = TaskDB()
    md = requests.get(REDITSOTA_URL).text

    # assumptions:
    # ### Category
    # #### Task
    md_lines = md.split("\n")

    category = None
    task = None
    for i in range(len(md_lines)):
        line = md_lines[i]

        if line.startswith("###") and not line.startswith("####"):
            category = line.replace("###", "").strip()

        if line.startswith("####") and not line.startswith("#####"):
            task = line.replace("####", "").strip()
            task = re.sub("^[0-9+].?", "", task).strip()

        if "<table>" in line.lower():
            end_i = None
            # find the end of table
            for j in range(i, len(md_lines)):
                if "</table>" in md_lines[j].lower():
                    end_i = j + 1
                    break

            if end_i and task and category:
                html_lines = md_lines[i:end_i]
                h = "\n".join(html_lines)

                soup = BeautifulSoup(h, "html.parser")

                # parse out the individual rows
                entries = []
                rows = soup.findAll("tr")
                for row in rows:
                    cells = row.findAll("td")
                    if len(cells) >= 4:
                        # paper ref
                        c_paper = cells[0]
                        paper_title = c_paper.text.strip()
                        paper_url = None
                        if c_paper.find("a"):
                            paper_url = c_paper.find("a")["href"]

                        # datasets
                        c_datasets = cells[1]
                        c_datasets_li = c_datasets.findAll("li")
                        dataset_names = []
                        for dataset_li in c_datasets_li:
                            dataset_names.append(dataset_li.text.strip())

                        # metrics
                        c_metrics = cells[2]
                        c_metrics_li = c_metrics.findAll("li")
                        metrics = []
                        for metrics_li in c_metrics_li:
                            parts = metrics_li.text.split(":")
                            parts = [p.strip() for p in parts]
                            m = {}
                            if len(parts) == 2:
                                m[parts[0]] = parts[1]
                                metrics.append(m)

                        if not metrics:
                            # Try to use it as single value
                            parts = c_metrics.text.split(":")
                            parts = [p.strip() for p in parts]
                            m = {}
                            if len(parts) == 2:
                                m[parts[0]] = parts[1]
                                metrics.append(m)

                        # source code ref
                        c_code = cells[3]
                        c_code_a = c_code.findAll("a")
                        code_links = []
                        for code_a in c_code_a:
                            code_links.append(
                                Link(
                                    title=code_a.text.strip(),
                                    url=code_a["href"],
                                )
                            )

                        entries.append(
                            {
                                "paper_title": paper_title,
                                "paper_url": paper_url,
                                "dataset_names": dataset_names,
                                "metrics": metrics,
                                "code_links": code_links,
                            }
                        )

                # Add the new task
                t = Task(name=task, categories=[category])
                t.source_link = Link(title="RedditSota", url=REDITSOTA_URL)

                # Add datasets and perfomance on them
                data_map = {}
                for e in entries:
                    if len(e["dataset_names"]) == len(e["metrics"]):
                        for j in range(len(e["dataset_names"])):
                            dataset_name = e["dataset_names"][j]
                            # make sure the dataset exists
                            if dataset_name not in data_map:
                                # collect all the metrics mentioned for this
                                # dataset
                                all_metrics = [
                                    list(ee["metrics"][j].keys())
                                    for ee in entries
                                    if dataset_name in ee["dataset_names"]
                                ]
                                all_metrics = [
                                    item
                                    for sublist in all_metrics
                                    for item in sublist
                                ]
                                all_metrics = list(set(all_metrics))
                                dataset = Dataset(
                                    name=dataset_name,
                                    is_subdataset=False,
                                    sota=Sota(metrics=all_metrics),
                                )
                                data_map[dataset_name] = dataset
                                t.datasets.append(dataset)
                            else:
                                dataset = data_map[dataset_name]

                            # record the metric for this dataset
                            sr = SotaRow(
                                model_name="",
                                paper_title=e["paper_title"],
                                paper_url=e["paper_url"],
                                metrics=e["metrics"][j],
                                code_links=e["code_links"],
                            )
                            dataset.sota.rows.append(sr)

                # add and reset the task
                tdb.add_task(t)
                task = None

    return tdb.export()



import io
import json
import enum
import gzip
from sota_extractor import errors


class Format(enum.Enum):
    """Output format.

    At the moment only supported format is JSON, but in the future YAML support
    is planned.
    """

    json = "json"
    json_gz = "json.gz"


def dump(data, filename, fmt=Format.json, encoding="utf-8"):
    """Write sota data to file.

    Intention of this helper function is to always have maximally similar
    json files after export. To do that it will always sort json object keys
    alphabetically, use the same indent, same encoding and same serializer.

    Args:
        data: Data for serialization.
        filename (str): Path to the file in which the data should be
            serialized.
        fmt (Format): Serialization format.
        encoding (str): File encoding.
    """
    if fmt == Format.json:
        with io.open(filename, mode="w", encoding=encoding) as fp:
            json.dump(data, fp=fp, indent=2, sort_keys=True)
    elif fmt == Format.json_gz:
        with gzip.open(filename, mode="wb") as fp:
            fp.write(
                json.dumps(data, fp=fp, indent=2, sort_keys=True).encode(
                    encoding
                )
            )
    else:
        raise errors.UnsupportedFormat(fmt)


def load(filename, fmt=Format.json, encoding="utf-8"):
    """Load sota data from file.

    Args:
        filename (str): Path to the file from which the data should be
            deserialized.
        fmt (Format): Serialization format.
        encoding (str): File encoding.
    """
    if fmt == Format.json:
        with io.open(filename, mode="r", encoding=encoding) as fp:
            return json.load(fp)
    elif fmt == Format.json_gz:
        with gzip.open(filename, mode="rb") as fp:
            return json.loads(fp.read().decode(encoding))
    else:
        raise errors.UnsupportedFormat(fmt)



__all__ = ["cli", "eff", "reddit", "snli", "squad", "evaluate"]

from sota_extractor.commands.cli import cli
from sota_extractor.commands.scrapers import eff, reddit, snli, squad
from sota_extractor.commands.evaluate import evaluate



import click
from sota_extractor import scrapers
from sota_extractor import serialization
from sota_extractor.commands.cli import cli
from sota_extractor.errors import catch_errors


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/eff.json",
    help="Output JSON filename to use.",
)
@catch_errors
def eff(output):
    """Extract EFF SOTA tables."""
    serialization.dump(scrapers.eff(), output)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/redditsota.json",
    help="Output JSON filename to use.",
)
@catch_errors
def reddit(output):
    """Extract Reddit SOTA tables."""
    serialization.dump(scrapers.reddit(), output)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/snli.json",
)
@catch_errors
def snli(output):
    """Extract SNLI SOTA tables."""
    serialization.dump(scrapers.snli(), output)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/tasks/squad.json",
    help="Output JSON filename to use.",
)
@catch_errors
def squad(output):
    """Extract SQUAD SOTA tables."""
    serialization.dump(scrapers.squad(), output)



import click
import functools
from requests import Response


class SotaError(Exception):
    def __init__(self, message):
        self.message = message

    @property
    def name(self):
        return self.__class__.__name__

    def __str__(self):
        return f"{self.name}(message={self.message})"

    __repr__ = __str__


class DataError(SotaError):
    pass


class ArgumentError(SotaError):
    pass


class HttpClientError(SotaError):
    def __init__(self, message, response=None):
        super().__init__(message)
        self.response: Response = response
        self.status_code = (
            response.status_code if response is not None else 500
        )

    def __str__(self):
        return (
            f"{self.name}(message={self.message}, "
            f"status_code={self.status_code})"
        )

    __repr__ = __str__


class UnsupportedFormat(SotaError):
    def __init__(self, fmt):
        super().__init__(f"Unsupported serialization format: {fmt.value}")
        self.fmt = fmt


def catch_errors(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except SotaError as e:
            click.secho(str(e), fg="red")
        except Exception as e:
            click.secho(f"Unknown error: {e}", fg="red")

    return wrapper



EFF_TASK_CONVERSION = dict(
    [
        ("Detection of Instrumentals musical tracks", "Music Autotagging"),
        (
            "Superhuman mastery of arbitrary abstract strategy games",
            "Abstract Strategy Games",
        ),
        (
            "Play an arbitrary abstract game, first learning the rules",
            "Abstract Strategy Games with Rule Learning",
        ),
        (
            "Solve technical problems with clear constraints (proofs, circuit "
            "design, aerofoil design, etc)",
            "Constrained Problem Solving",
        ),
        (
            "Know how to prevent an autonomous AI agent from reproducing "
            "itself an unbounded number of times",
            "Restricted Reproduction",
        ),
        (
            "Transfer of learning within simple arcade game paradigms",
            "Arcade Game Transfer Learning",
        ),
        (
            "Train machine learning systems on private user data, without "
            "transferring sensitive facts into the model",
            "Privacy Preserving Machine Learning",
        ),
        ("Avoiding reward hacking", "Reward Hacking Avoidance"),
        ("Drawing pictures", "Image Generation"),
        ("Recognise events in videos", "Video Activity Recognition"),
        (
            "Correctly identify when an answer to a classification problem is "
            "uncertain",
            "Classification Under Uncertainty",
        ),
        (
            "Deploy automated defensive security tools to protect valuable "
            "systems",
            "Automated Security",
        ),
        (
            "Scalable supervision of a learning system",
            "Scalable Supervised Learning",
        ),
        ("Pedestrian, bicycle & obstacle detection", "Pedestrian Detection"),
        (
            "Given desired circuit characteristics, and many examples, design "
            "new circuits to spec",
            "Automated Circuit Design",
        ),
        (
            "One shot learning, ingest important truths from a single example",
            "One-Shot Learning",
        ),
        (
            "Language comprehension and question-answering",
            "Question Answering",
        ),
        (
            "Given examples of proofs, find correct proofs of simple "
            "mathematical theorems",
            "Mathematical Proofs",
        ),
        ("Games that require language comprehension", "Language Games"),
        (
            "Games that require both understanding and speaking a language",
            "Spoken Language Games",
        ),
        (
            "Learn a several tasks without undermining performance on a first "
            "task, avoiding catastrophic forgetting",
            "Catastrophic Forgetting Avoidance",
        ),
        (
            "Train ML classifiers in a manner that corrects for the impact of "
            "omitted-variable bias on certain groups",
            "Omitted-Variable Bias Correction",
        ),
        (
            "Given an arbitrary technical problem, solve it as well as a "
            "typical professional in that field",
            "Artificial General Intelligence",
        ),
        ("Writing software from specifications", "Code Generation"),
        ("Translation between human langauges", "Machine Translation"),
        (
            "Fairness in machine learning towards people with a preference "
            "for privacy",
            "Privacy Fairness",
        ),
        ("Safe exploration", "Safe Exploration"),
        (
            "Turing test for casual conversation",
            "Casual Conversation Turing Test",
        ),
        ("Write computer programs from specifications", "Code Generation"),
        (
            "Transfer learning, apply relevant knowledge from a prior setting "
            "to a new slightly different one",
            "Transfer Learning",
        ),
        ("Play real-time computer & video games", "Video Games"),
        (
            "Extract major numerical results or progress claims from a STEM "
            "paper",
            "Scientific Result Extraction",
        ),
        ("Abstract strategy games", "Abstract Strategy Games"),
        (
            "Cooperative inverse reinforcement learning of objective "
            "functions",
            "Objective Function Reinforcement Learning",
        ),
        (
            "Provide mathematical or technical explanations of decisions "
            "from classifiers",
            "Explainable Machine Learning",
        ),
        ("Image classification", "Image Classification"),
        (
            "Read a scientific or technical paper, and comprehend its "
            "contents",
            "Scientific Paper Comprehension",
        ),
        ("Answering Science Exam Questions", "Scientific Question Answering"),
        (
            "Conduct arbitrary sustained, probing conversation",
            "Automated Interrogation",
        ),
        (
            "Building systems that solve a wide range of diverse problems, "
            "rather than just specific ones",
            "Artificial General Intelligence",
        ),
        (
            "Modify arbitrary ML systems in order to be able to provide "
            "comprehensible human explanations of their decisions",
            "Explainable Machine Learning",
        ),
        (
            "Learning the rules of complex strategy games from examples",
            "Strategy Game Rule Learning",
        ),
        (
            "Know how to build general AI agents that will behave as expected",
            "Predictable Artificial General Intelligence",
        ),
        (
            "Build systems which can recognise and avoid biases decision "
            "making",
            "Bias Avoidance",
        ),
        (
            "Function correctly in novel environments (robustness to "
            "distributional change)",
            "Functional Robustness",
        ),
        (
            "Detect security-related bugs in codebases",
            "Security Bug Detection",
        ),
        ("Simple video games", "Simple Video Games"),
        (
            "Playing abstract games with extensive hints",
            "Abstract Games with Hints",
        ),
        ("Accurate modelling of human language.", "Language Modelling"),
        (
            "Games that require inventing novel language, forms of speech, or "
            "communication",
            "Language Creation Games",
        ),
        ("Resistance to adversarial examples", "Adversarial Defense"),
        (
            "Parse and implement complex conditional expressions",
            "Conditional Expression Parsing",
        ),
        ("Image comprehension", "Visual Question Answering"),
        (
            "Be able to generate complex scene e.g. a baboon receiving their "
            "degree at convocatoin.",
            "Complex Conditional Image Generation",
        ),
        (
            "Solve vaguely or under-constrained technical problems",
            "Underconstrained Problem Solving",
        ),
        ("Avoiding undesirable side effects", "Side Effect Mitigation"),
    ]
)



import io
import csv
from typing import Dict, List, Optional
from sota_extractor import serialization
from sota_extractor.errors import ArgumentError
from sota_extractor.taskdb.v01.models import Task, Dataset
from sota_extractor.taskdb.v01.schemas import TaskSchema


class TaskDB:
    def __init__(self):
        self.tasks = {}
        self.schema = TaskSchema()

    def get_task(self, name: str) -> Optional[Task]:
        """Get a task or a sub-task by name."""
        if name in self.tasks:
            return self.tasks[name]
        else:
            for task in self.tasks.values():
                for subtask in task.subtasks:
                    if subtask.name == name:
                        return subtask
        return None

    def add_task(self, task: Task):
        """Add a top-level task by name."""
        self.tasks[task.name] = task

    def load_tasks(self, files: List[str] = None, data: List[Dict] = None):
        """Load tasks from files or from data.

        Args:
            files (List[str] | str): Path to a document or a list of paths to
                documents.
            data: Sota data - list of dictionaries representing tasks.
        """
        if files is None and data is None:
            raise ArgumentError("Either 'files' or 'data' must be supplied.")

        if isinstance(files, str):
            files = [files]

        if files is not None:
            data = []
            for file in files:
                data.extend(serialization.load(file))

        task_list = self.schema.load(data, many=True)
        for task in task_list:
            self.add_task(task)

    def load_synonyms(self, csv_files: List[str]):
        """Load task synonyms from input files."""
        if isinstance(csv_files, str):
            csv_files = [csv_files]
        for csv_file in csv_files:
            with io.open(csv_file, newline="") as f:
                reader = csv.reader(f)
                for row in reader:
                    task = self.get_task(row[0])
                    if task is not None:
                        task.synonyms.append(row[1])

    def tasks_with_sota(self) -> List[Task]:
        """Extract all tasks with SOTA tables.

        This includes both the top-level and sub-tasks.
        """
        sota_tasks = []
        for task in self.tasks.values():
            find_sota_tasks(task, sota_tasks)
        return sota_tasks

    def datasets_with_sota(self) -> List[Dataset]:
        """Extract all datasets with SOTA tables.

        This includes both the top-level and sub-tasks.
        """

        sota_datasets = []

        for task in self.tasks.values():
            find_sota_datasets(task, sota_datasets)
        return sota_datasets

    def export(self) -> List[Dict]:
        """Export the whole of TaskDB into a list of tasks in Dict format."""
        return self.schema.dump(self.tasks.values(), many=True)

    def export_to_file(self, filename: str, fmt=serialization.Format.json):
        """Export the whole of TaskDB into a file."""
        serialization.dump(self.export(), filename=filename, fmt=fmt)


def find_sota_tasks(task: Task, out: List):
    """Get all the tasks with a SOTA table.

    These tasks will be added into the "out" output list.
    """
    # check if the dataset has sota tables
    add = False
    for d in task.datasets:
        if d.sota.rows:
            add = True

        for sd in d.subdatasets:
            if sd.sota.rows:
                add = True
    if add:
        out.append(task)

    for subtask in task.subtasks:
        find_sota_tasks(subtask, out)


def find_sota_datasets(task: Task, out: List):
    """Get all the datasets with a SOTA table.

    These datasets will be added into the "out" output list.
    """

    # check if the dataset has sota tables
    add = False
    for d in task.datasets:
        if d.sota.rows:
            add = True

        for sd in d.subdatasets:
            if sd.sota.rows:
                add = True

        if add:
            out.append(d)

    for subtask in task.subtasks:
        find_sota_datasets(subtask, out)



import requests
from bs4 import BeautifulSoup


def get_soup(url):
    """Get a BeautifulSoup object back from the a URL.

    Args:
        url: URL to scrape.
    """

    r = requests.get(url)

    if r.status_code == 404:
        return None

    data = r.text
    return BeautifulSoup(data, "lxml")



__all__ = ["eff", "snli", "squad", "reddit"]


from sota_extractor.scrapers.eff import eff
from sota_extractor.scrapers.snli import snli
from sota_extractor.scrapers.squad import squad
from sota_extractor.scrapers.reddit import reddit



import json
import requests
from sota_extractor.errors import HttpClientError
from sota_extractor.scrapers.consts import EFF_TASK_CONVERSION
from sota_extractor.taskdb.v01 import (
    Task,
    Dataset,
    Link,
    SotaRow,
    Sota,
    TaskDB,
)

EFF_URL = (
    "https://raw.githubusercontent.com/AI-metrics/AI-metrics/master/"
    "export-api/v01/progress.json"
)


def eff():
    """Extract EFF SOTA tables."""

    response = requests.get(EFF_URL)
    if response.status_code != 200:
        raise HttpClientError("Resource unavailable", response=response)
    j = json.loads(response.text)
    tdb = TaskDB()

    for problem in j["problems"]:

        if problem["name"] in EFF_TASK_CONVERSION:
            problem_name = EFF_TASK_CONVERSION[problem["name"]]
        else:
            problem_name = problem["name"]

        task = Task(name=problem_name)

        task.source_link = Link(
            title="Progress of AI Research",
            url="https://github.com/AI-metrics/AI-metrics",
        )

        datasets = []
        for metric in problem["metrics"]:
            if "measures" in metric and metric["measures"]:
                measures = metric["measures"]

                dataset = Dataset(
                    name=metric["name"],
                    is_subdataset=False,
                    sota=Sota(metrics=[metric["scale"]]),
                )

                for measure in measures:
                    sr = SotaRow(
                        model_name=measure["name"],
                        paper_title=measure["papername"],
                        paper_url=measure["url"],
                        metrics={metric["scale"]: measure["value"]},
                    )

                    if measure["replicated_url"]:
                        sr.code_links.append(
                            Link(
                                title="Replicated",
                                url=measure["replicated_url"],
                            )
                        )

                    dataset.sota.rows.append(sr)

                datasets.append(dataset)

        task.datasets = datasets
        tdb.add_task(task)

    return tdb.export()



import re
import click
import pandas as pd
from typing import List, Dict
from nltk.stem.porter import PorterStemmer
from sota_extractor import serialization
from sota_extractor.commands.cli import cli
from sota_extractor.errors import catch_errors
from sota_extractor.taskdb.v01 import Task, TaskDB


def load(tdb):
    # load the tasks and arxiv metadata
    stemmer = PorterStemmer()

    tdb.load_tasks("data/tasks/nlpprogress.json")
    tdb.load_synonyms(["data/tasks/synonyms.csv"])
    arxiv = serialization.load(
        "data/arxiv_aclweb.json.gz", fmt=serialization.Format.json_gz
    )

    for a in arxiv:
        if a["abstract"] is None:
            a["abstract"] = ""

    # require and normalise arxiv titles
    arxiv = [a for a in arxiv if "title" in a and a["title"] is not None]
    for a in arxiv:
        a["title"] = re.sub(" +", " ", a["title"].replace("\n", " "))
        a["title_lower"] = a["title"].lower()
        a["abstract_lower"] = a["abstract"].lower()
        a["title_stem"] = stemmer.stem(a["title"])
        a["abstract_stem"] = stemmer.stem(a["abstract"])

    return arxiv


def eval_task(predicted: List[Dict], task: Task):
    """Get the precision and recall for a task, using any dataset.

    Args:
        predicted: The set of predicted true positive papers.
        task: The task we are doing predictions against.
    """

    all_sota = []
    for d in task.datasets:
        all_sota.extend(d.sota.rows)
        for sd in d.subdatasets:
            all_sota.extend(sd.sota.rows)

    # true positives
    tp = []
    tp_sota = []
    for p in predicted:
        for s in all_sota:
            if (
                p["arxiv_id"] and s.paper_url and p["arxiv_id"] in s.paper_url
            ) or (
                p["title_lower"]
                and s.paper_title
                and p["title_lower"] == s.paper_title.lower()
            ):
                tp.append(p)
                tp_sota.append(s)

    fn = [s for s in all_sota if s not in tp_sota]
    fp = [p for p in predicted if p not in tp]

    return tp, fn, fp


def article_matches(paper: Dict, task: Task):
    """Check if a paper mentions the tasks.

    By mentioning it in the title or abstract.
    And also mentioning state-of-the-art.
    """

    title = paper["title_lower"]
    abstract = paper["abstract_lower"]

    all_task_names = [task.name]
    all_task_names.extend(task.synonyms)

    matches_paper = False
    for task_name in all_task_names:
        task_name_lower = task_name.lower()
        matches_paper = matches_paper or task_name_lower in title
        matches_paper = matches_paper or task_name_lower in abstract

    contains_sota = (
        "state-of-the-art" in abstract
        or "state-of-art" in abstract
        or "state of the art" in abstract
        or "state of art" in abstract
        or "sota" in abstract
    )

    return matches_paper and contains_sota


def eval_all(tdb, arxiv, output):
    sota_tasks = tdb.tasks_with_sota()

    df = pd.DataFrame(
        columns=["task", "parent", "tp", "fn", "fp", "precision", "recall"]
    )

    for task in sota_tasks:
        pred = [a for a in arxiv if article_matches(a, task)]
        tp, fn, fp = eval_task(pred, task)

        prec = 0
        recal = 0
        if (len(tp) + len(fp)) != 0:
            prec = len(tp) / (len(tp) + len(fp))
        if len(tp) / (len(tp) + len(fn)) != 0:
            recal = len(tp) / (len(tp) + len(fn))

        parent = ""
        if task.parent:
            parent = task.parent.name

        df = df.append(
            {
                "task": task.name,
                "parent": parent,
                "tp": len(tp),
                "fn": len(fn),
                "fp": len(fp),
                "precision": round(prec, 2),
                "recall": round(recal, 2),
            },
            ignore_index=True,
        )

    df = df.append(
        {
            "task": "",
            "parent": "Total",
            "tp": round(df["tp"].mean(), 2),
            "fn": round(df["fn"].mean(), 2),
            "fp": round(df["fp"].mean(), 2),
            "precision": round(df["precision"].mean(), 2),
            "recall": round(df["recall"].mean(), 2),
        },
        ignore_index=True,
    )

    click.echo(f"Writing report into: {output}")
    df.to_csv(output)


@cli.command()
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False),
    required=False,
    default="data/eval_all_report.csv",
    help="Output filename to use.",
)
@catch_errors
def evaluate(output):
    """Evaluate."""
    tdb = TaskDB()
    arxiv = load(tdb)
    eval_all(tdb, arxiv, output)



__all__ = ["Link", "SotaRow", "Sota", "Dataset", "Task", "TaskDB"]

from sota_extractor.taskdb.v01.models import Link, SotaRow, Sota, Dataset, Task
from sota_extractor.taskdb.v01.taskdb import TaskDB



import io
from datetime import datetime
from setuptools import setup, find_packages


setup(
    name="sota-extractor",
    version="0.0.8",
    description="Automatic SOTA (state-of-the-art) extraction.",
    long_description=io.open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    platforms=["Windows", "POSIX", "MacOS"],
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3",
    ],
    keywords="sota table",
    author="Robert Stojnic",
    author_email="robert@atlasml.io",
    maintainer="Viktor Kerkez",
    maintainer_email="viktor@atlasml.io",
    url="https://github.com/paperswithcode/sota-extractor",
    license="Apache-2.0",
    packages=find_packages(),
    install_requires=io.open("requirements.txt").read().splitlines(),
    include_package_data=True,
    scripts=[],
    entry_points="""
        [console_scripts]
        sota-extractor=sota_extractor.__main__:cli
    """,
)
